package src.data.scripts.campaign.missions;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.characters.FullName;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.impl.campaign.events.OfficerManagerEvent;
import java.util.List;
import src.data.scripts.campaign.GladiatorSociety_Content;
import src.data.scripts.campaign.dataclass.GladiatorSociety_BountyData;
import src.data.scripts.campaign.dataclass.GladiatorSociety_DataShip;

public class GladiatorSociety_BountyMission_V2  {

    private final GladiatorSociety_Content parent;
    private final int reward;
    private float duration = 1f;
    private final String factionId;
    private PersonAPI person;

    private final GladiatorSociety_BountyData bountydata;
    private SectorEntityToken hideoutLocation = null;

    public GladiatorSociety_BountyMission_V2(String factionId, GladiatorSociety_BountyData entity, float duration, GladiatorSociety_Content parent) {
        this.bountydata = entity;
        this.reward = entity.bountyvalue;
        this.duration = duration;
        initPerson();
        this.factionId = factionId;
        this.parent = parent;

        for (SectorEntityToken token : Global.getSector().getPlayerFleet().getContainingLocation().getAllEntities()) {
            if (token.hasTag("comm_relay")) {
                this.hideoutLocation = token;
                break;
            }
        }
        if (this.hideoutLocation == null && !Global.getSector().getPlayerFleet().getContainingLocation().getAllEntities().isEmpty()) {
            hideoutLocation = Global.getSector().getPlayerFleet().getContainingLocation().getAllEntities().get(0);
        }
        //this.hideoutLocation = Global.getSector().getPlayerFleet().getInteractionTarget();
        //CampaignEventManagerAPI eventManager = Global.getSector().getEventManager();
        //CampaignEventTarget target = new CampaignEventTarget(hideoutLocation);
       // target.setExtra(Misc.genUID());

    }

    public GladiatorSociety_Content getCreator() {
        return this.parent;
    }
    public int getSupposedFleetPoints(){
        return this.bountydata.fleetPoints;
    }

    public static final String CUSTOMPORTRAIT = "customportrait";

    private void initPerson() {
        person = OfficerManagerEvent.createOfficer(Global.getSector().getFaction(bountydata.faction), this.bountydata.flagship.number,true);
        // person = OfficerManagerEvent.createOfficer(Global.getSector().getFaction(GladiatorSociety_Constants.GSFACTION_ID), personLevel);
        if (!bountydata.avatar.isEmpty()) {
            person.setPortraitSprite(bountydata.avatar);

            if (person.getPortraitSprite() == null) {
                person.setFaction(CUSTOMPORTRAIT);
                person.setPortraitSprite(bountydata.avatar);
                person.setFaction(bountydata.faction);
            }

        }
        if (bountydata.firstname.isEmpty() || bountydata.lastname.isEmpty()) {
            return;
        }

        person.setPersonality( this.bountydata.flagship.personality);
        person.setName(new FullName(bountydata.firstname, bountydata.lastname, bountydata.gender));
    }

    public PersonAPI getPerson() {
        return this.person;
    }

    public SectorEntityToken getLocation() {
        return this.hideoutLocation;
    }

    public int getCombatPoints() {
        return this.bountydata.combatPoint;
    }

    public int getDynasectorSettings() {
        return this.bountydata.dsrandom;
    }
/*
    public String getPostingStage() {
        return "posting";
    }*/
/*
    public String getName() {
        String where = this.hideoutLocation.getContainingLocation().getNameWithTypeIfNebula();

        return "G-Match in " + where;
    }*/

    public String getBountyFaction() {
        return this.bountydata.faction;
    }

    public String getBountyDescription() {
        return this.bountydata.description;
    }

    public List<GladiatorSociety_DataShip> getBountyShips() {
        return this.bountydata.advships;
    }

    public boolean getBountyRandom() {
        return this.bountydata.randomFleet;
    }

    public GladiatorSociety_DataShip getBountyMotherShip() {
        return this.bountydata.flagship;
    }

    public String getDialog() {
        return this.bountydata.dialog;
    }

    public String getMissionID() {
        return this.bountydata.missionid;
    }

    public boolean canPlayerAccept() {
        return true;
    }

    public void playerAccept(SectorEntityToken entity) {
        GladiatorSociety_GladiatorManager.getInstance().createEvent(this);
    }
    public int getReward() {
        return reward;
    }

    public float getDuration() {
        return duration;
    }

    public String getFactionId() {
        return factionId;
    }

}
