package src.data.scripts.campaign.missions;

import java.awt.Color;
import java.util.Set;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetAssignment;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.CampaignEventListener.FleetDespawnReason;
import com.fs.starfarer.api.campaign.ReputationActionResponsePlugin.ReputationAdjustmentResult;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.comm.IntelInfoPlugin;
import com.fs.starfarer.api.campaign.listeners.FleetEventListener;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.characters.FullName.Gender;
import com.fs.starfarer.api.impl.campaign.events.OfficerManagerEvent;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.impl.campaign.ids.Abilities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.intel.BaseIntelPlugin;
import com.fs.starfarer.api.impl.campaign.procgen.Constellation;
import com.fs.starfarer.api.impl.campaign.shared.PersonBountyEventData;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import com.fs.starfarer.api.plugins.OfficerLevelupPlugin;
import com.fs.starfarer.api.ui.SectorMapAPI;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;
import src.data.scripts.campaign.GladiatorSociety_EndlessContent;
import src.data.scripts.campaign.GladiatorSociety_TinyFleetFactoryV2;
import src.data.utils.GladiatorSociety_Constants;

public class GladiatorSociety_EventEndlessBattle_v2 extends BaseIntelPlugin implements EveryFrameScript, FleetEventListener {

    public static Logger log = Global.getLogger(GladiatorSociety_EventEndlessBattle_v2.class);

    private float elapsedDays = 0;
    private boolean ended = false;
    private GladiatorSociety_EndlessContent content;
    private SectorEntityToken relay;
    private FactionAPI faction;
    private PersonAPI person;
    private int level;

    private CampaignFleetAPI fleet;

    public float getElapsedDays() {
        return elapsedDays;
    }

    public void setElapsedDays(float elapsedDays) {
        this.elapsedDays = elapsedDays;
    }

    public static PersonBountyEventData getSharedData() {
        return SharedData.getData().getPersonBountyEventData();
    }

    public GladiatorSociety_EventEndlessBattle_v2(GladiatorSociety_EndlessContent content) {
        this.content=content;

        for (SectorEntityToken token : Global.getSector().getPlayerFleet().getContainingLocation().getAllEntities()) {
            if (token.hasTag("comm_relay")) {
                this.relay = token;
                break;
            }
        }
        if (this.relay == null && !Global.getSector().getPlayerFleet().getContainingLocation().getAllEntities().isEmpty()) {
            relay = Global.getSector().getPlayerFleet().getContainingLocation().getAllEntities().get(0);
        }
        pickFaction();
        if (isDone()) {
            return;
        }
        initPerson();
        if (isDone()) {
            return;
        }
        spawnFleet();
        if (isDone()) {
            return;
        }
        SectorAPI sector = Global.getSector();
        FactionAPI player = sector.getFaction(Factions.PLAYER);
        player.setRelationship(GladiatorSociety_Constants.GSFACTION_ID, -1);
        log.info(String.format("Endless Battle begin ", content.getEndlessFaction()));
       // Global.getSector().getIntelManager().queueIntel(this);
         Global.getSector().getIntelManager().addIntel(this, true);
        
      //  Global.getSector().getIntelManager().add
    }
    
   

    @Override
    public void reportMadeVisibleToPlayer() {
    }

    public int getLevel() {
        return level;
    }

    private void pickFaction() {
        faction = Global.getSector().getFaction(content.getEndlessFaction());
        if (faction == null) {
            this.ended = true;
        }
    }

    private void initPerson() {
        level = ((OfficerLevelupPlugin) Global.getSettings().getPlugin("officerLevelUp")).getMaxLevel(null);
        int personLevel = (int) (5 + level * 1.5f);
        person = OfficerManagerEvent.createOfficer(Global.getSector().getFaction(faction.getId()),
                personLevel, true);
    }

    @Override
    protected void advanceImpl(float amount) {
        float days = Global.getSector().getClock().convertToDays(amount);
        //days *= 60f;

        elapsedDays += days;

        if ((100 - elapsedDays <= 0) && !isDone()) {
            boolean canEnd = fleet == null || !fleet.isInCurrentLocation();
            if (canEnd) {
                log.info(String.format("Ending Endless Battle"));
                result = new BountyResult(BountyResultType.END_TIME, 0, null);
                sendUpdateIfPlayerHasIntel(result, true);
                cleanUpFleetAndEndIfNecessary();
                return;
            }
        }

        if (fleet == null) {
            return;
        }

        if (fleet.getFlagship() == null || fleet.getFlagship().getCaptain() != person) {
            result = new BountyResult(BountyResultType.END_OTHER, 0, null);
            boolean current = fleet.isInCurrentLocation();
            sendUpdateIfPlayerHasIntel(result, !current);
            cleanUpFleetAndEndIfNecessary();
            return;
        }
    }

    @Override
    public float getTimeRemainingFraction() {
        float f = 1f - elapsedDays / 100;
        return f;
    }

    protected BountyResult result = null;

    @Override
    protected void notifyEnding() {
        super.notifyEnding();
        cleanUpFleetAndEndIfNecessary();
    }

    protected void cleanUpFleetAndEndIfNecessary() {
        if (fleet != null) {
            Misc.makeUnimportant(fleet, "pbe");
            fleet.clearAssignments();
            if (relay != null) {
                fleet.getAI().addAssignment(FleetAssignment.GO_TO_LOCATION_AND_DESPAWN, relay, 1000000f, null);
            } else {
                fleet.despawn();
            }
            fleet = null; //can't null it because description uses it
        }
        if (!isEnding() && !isEnded()) {
            endAfterDelay();
        }
    }

    protected boolean willPay() {
        return true;

    }

    protected boolean willRepIncrease() {
        return false;

    }

    @Override
    public void reportBattleOccurred(CampaignFleetAPI fleet, CampaignFleetAPI primaryWinner, BattleAPI battle) {
        if (isDone() || result != null) {
            return;
        }

        if (battle.isInvolved(fleet) && !battle.isPlayerInvolved()) {
            if (fleet.getFlagship() == null || fleet.getFlagship().getCaptain() != person) {
                fleet.setCommander(fleet.getFaction().createRandomPerson());
                //Global.getSector().reportEventStage(this, "other_end", market.getPrimaryEntity(), messagePriority);
                result = new BountyResult(BountyResultType.END_OTHER, 0, null);
                sendUpdateIfPlayerHasIntel(result, true);
                cleanUpFleetAndEndIfNecessary();
                return;
            }
        }

        CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
        if (!battle.isPlayerInvolved() || !battle.isInvolved(fleet) || battle.onPlayerSide(fleet)) {
            return;
        }

        // didn't destroy the original flagship
        if (fleet.getFlagship() != null && fleet.getFlagship().getCaptain() == person) {
            return;
        }

        int payment = (int) (content.getEndlessReward() * battle.getPlayerInvolvementFraction());
        if (payment <= 0) {
            result = new BountyResult(BountyResultType.END_OTHER, 0, null);
            sendUpdateIfPlayerHasIntel(result, true);
            cleanUpFleetAndEndIfNecessary();
            return;
        }

        log.info(String.format("Paying salary of %d from Gladiator Society", (int) payment, faction.getDisplayName()));
	Global.getSector().getIntelManager().removeIntel(this);

        playerFleet.getCargo().getCredits().add(payment);
       /* ReputationAdjustmentResult rep = Global.getSector().adjustPlayerReputation(
                new RepActionEnvelope(RepActions.PERSON_BOUNTY_REWARD, null, null, null, true, false),
                faction.getId());*/
        result = new BountyResult(BountyResultType.END_PLAYER_BOUNTY, payment, null);
        sendUpdateIfPlayerHasIntel(result, false);

        getSharedData().reportSuccess();
        content.incEndlessRound();

        //removeBounty();

        cleanUpFleetAndEndIfNecessary();
    }

    @Override
    public void reportFleetDespawnedToListener(CampaignFleetAPI fleet, FleetDespawnReason reason, Object param) {
        if (isDone() || result != null) {
            return;
        }

        if (this.fleet == fleet) {
            fleet.setCommander(fleet.getFaction().createRandomPerson());
            result = new BountyResult(BountyResultType.END_OTHER, 0, null);
            sendUpdateIfPlayerHasIntel(result, true);
            cleanUpFleetAndEndIfNecessary();
        }
    }

    private void spawnFleet() {
        
        float random = (int) (Math.random() * 10) / 10f;
      

        FleetParamsV3 params = new FleetParamsV3(
                null,
                relay.getLocationInHyperspace(),
                faction.getId(), // quality will always be reduced by non-market-faction penalty, which is what we want 
                null,
                FleetTypes.PERSON_BOUNTY_FLEET,
                content.getEndlessPower() + random, // combatPts
                0, // freighterPts 
                0, // tankerPts
                0f, // transportPts
                0f, // linerPts
                0f, // utilityPts
                1f // qualityMod
        );
        params.ignoreMarketFleetSizeMult = true;

        fleet = GladiatorSociety_TinyFleetFactoryV2.createFleet(params);
        if (fleet == null || fleet.isEmpty()) {
            endImmediately();
            return;
        }

             fleet.setCommander(person);
        fleet.getFlagship().setCaptain(person);
        FleetFactoryV3.addCommanderSkills(person, fleet, null);

        Misc.makeImportant(fleet, GladiatorSociety_Constants.GSFACTION_ID, 120);
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);
                fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_NO_MILITARY_RESPONSE, true);

        fleet.setNoFactionInName(true);
        
        fleet.setFaction(GladiatorSociety_Constants.GSFACTION_ID, true);
        
        
        fleet.addEventListener(this);

        LocationAPI location = Global.getSector().getPlayerFleet().getContainingLocation();
        location.addEntity(fleet);
        fleet.setName("Gladiator fleet");
        fleet.removeAbility(Abilities.INTERDICTION_PULSE);
        fleet.setLocation(Global.getSector().getPlayerFleet().getLocation().x, Global.getSector().getPlayerFleet().getLocation().y);
        fleet.getAI().addAssignment(FleetAssignment.INTERCEPT, Global.getSector().getPlayerFleet(), 1000000f, null);
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_MAKE_AGGRESSIVE, true);

        fleet.getMemoryWithoutUpdate().set("$dialog", "The gladiator glares at you briefly before shutting down the comm link.");

   
        
    }

    @Override
    public boolean runWhilePaused() {
        return false;
    }

    public static enum BountyResultType {
        END_PLAYER_BOUNTY,
        END_PLAYER_NO_BOUNTY,
        END_PLAYER_NO_REWARD,
        END_OTHER,
        END_TIME,
    }

    public static class BountyResult {

        public BountyResultType type;
        public int payment;
        public ReputationAdjustmentResult rep;

        public BountyResult(BountyResultType type, int payment, ReputationAdjustmentResult rep) {
            this.type = type;
            this.payment = payment;
            this.rep = rep;
        }
    }

    protected void addBulletPoints(TooltipMakerAPI info, IntelInfoPlugin.ListInfoMode mode) {

        Color h = Misc.getHighlightColor();
        Color g = Misc.getGrayColor();
        float pad = 3f;
        float opad = 10f;

        float initPad = pad;
        if (mode == IntelInfoPlugin.ListInfoMode.IN_DESC) {
            initPad = opad;
        }

        Color tc = getBulletColorForMode(mode);

        bullet(info);
        boolean isUpdate = getListInfoParam() != null;

        if (result == null) {
            if (mode == IntelInfoPlugin.ListInfoMode.IN_DESC) {

                info.addPara("%s reward", initPad, tc, h, Misc.getDGSCredits(content.getEndlessReward()));
//				}
                addDays(info, "remaining", 120 - elapsedDays, tc);
            } else {
                info.addPara("Faction: " + faction.getDisplayName(), initPad, tc,
                        faction.getBaseUIColor(), faction.getDisplayName());
                if (!isEnding()) {
                    int days = (int) (120 - elapsedDays);
                    String daysStr = "days";
                    if (days <= 1) {
                        days = 1;
                        daysStr = "day";
                    }
                    info.addPara("%s reward, %s " + daysStr + " remaining", 0f, tc,
                            h, Misc.getDGSCredits(content.getEndlessReward()), "" + days);
                }
            }
            unindent(info);
            return;
        }

        switch (result.type) {
            case END_PLAYER_BOUNTY:
                info.addPara("%s received", initPad, tc, h, Misc.getDGSCredits(result.payment));
               // CoreReputationPlugin.addAdjustmentMessage(result.rep.delta, faction, null,
              //          null, null, info, tc, isUpdate, 0f);
                break;
            case END_PLAYER_NO_BOUNTY:
               // CoreReputationPlugin.addAdjustmentMessage(result.rep.delta, faction, null,
               //         null, null, info, tc, isUpdate, 0f);
                break;
            case END_PLAYER_NO_REWARD:
               // CoreReputationPlugin.addAdjustmentMessage(result.rep.delta, faction, null,
               //         null, null, info, tc, isUpdate, 0f);
                break;
            case END_TIME:
                break;
            case END_OTHER:
                break;

        }

        unindent(info);
    }

    @Override
    public void createIntelInfo(TooltipMakerAPI info, IntelInfoPlugin.ListInfoMode mode) {
        Color h = Misc.getHighlightColor();
        Color g = Misc.getGrayColor();
        Color c = getTitleColor(mode);
        float pad = 3f;
        float opad = 10f;

        info.addPara(getName(), c, 0f);

        addBulletPoints(info, mode);

    }

    public String getSortString() {
        return "Endless Battle";
    }

    public String getName() {
        String n = person.getName().getFullName();

        if (result != null) {
            switch (result.type) {
                case END_PLAYER_BOUNTY:
                case END_PLAYER_NO_BOUNTY:
                case END_PLAYER_NO_REWARD:
                    return "Battle Completed - " + n;
                case END_OTHER:
                case END_TIME:
                    return "Battle Ended - " + n;
            }
        }

        return "Endless Battle - " + n;
    }

    @Override
    public FactionAPI getFactionForUIColors() {
        return faction;
    }

    @Override
    public String getSmallDescriptionTitle() {
        return getName();
        //return null;
    }

    @Override
    public void createSmallDescription(TooltipMakerAPI info, float width, float height) {
        Color h = Misc.getHighlightColor();
        Color g = Misc.getGrayColor();
        float pad = 3f;
        float opad = 10f;

        //info.addPara(getName(), c, 0f);
        //info.addSectionHeading(getName(), Alignment.MID, 0f);
        info.addImage(Global.getSettings().getSpriteName("campaignMissions", "gsbounty"), width, 128, opad);
         

        if (result != null) {
            if (result.type == BountyResultType.END_PLAYER_BOUNTY) {
                info.addPara("You have successfully completed this battle.", opad);
            } else if (result.type == BountyResultType.END_PLAYER_NO_BOUNTY) {
                info.addPara("You have successfully completed this battle, but received no "
                        + "credit reward because obvious reason. ",opad);
            } else if (result.type == BountyResultType.END_PLAYER_NO_REWARD) {
                info.addPara("You have successfully completed this bounty, but received no "
                        + "credit reward because obvious reason.",opad);
            } else {
                info.addPara("This battle is no longer on offer. (You are serious?)", opad);
            }
        } 

        addBulletPoints(info, IntelInfoPlugin.ListInfoMode.IN_DESC);
        if (result == null) {

            if (relay != null) {
                SectorEntityToken fake = relay.getContainingLocation().createToken(0, 0);
                String sheIs = "She is";
                if (person.getGender() == Gender.MALE) {
                    sheIs = "He is";
                }
                info.addPara(sheIs + " where you have requested the battle.", opad);
            }
        }
    }

    @Override
    public String getIcon() {
        return Global.getSettings().getSpriteName("campaignMissions", "gsbounty");
    }

    @Override
    public Set<String> getIntelTags(SectorMapAPI map) {
        Set<String> tags = super.getIntelTags(map);
        tags.add(GladiatorSociety_Constants.ENDLESSBATTLE);
        tags.add(GladiatorSociety_Constants.GSFACTION_ID);
        return tags;
    }

    @Override
    public SectorEntityToken getMapLocation(SectorMapAPI map) {
        Constellation c = relay.getConstellation();
        SectorEntityToken entity = null;
        if (c != null && map != null) {
            entity = map.getConstellationLabelEntity(c);
        }
        if (entity == null) {
            entity = relay;
        }
        return entity;
    }

}
