package src.data.scripts.campaign.missions;

import java.awt.Color;
import java.util.Set;

import org.apache.log4j.Logger;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.FleetAssignment;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.CampaignEventListener.FleetDespawnReason;
import com.fs.starfarer.api.campaign.ReputationActionResponsePlugin.ReputationAdjustmentResult;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.comm.IntelInfoPlugin;
import com.fs.starfarer.api.campaign.listeners.FleetEventListener;
import com.fs.starfarer.api.characters.FullName.Gender;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.impl.campaign.ids.Abilities;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.intel.BaseIntelPlugin;
import com.fs.starfarer.api.impl.campaign.procgen.Constellation;
import com.fs.starfarer.api.impl.campaign.shared.PersonBountyEventData;
import com.fs.starfarer.api.impl.campaign.shared.SharedData;
import com.fs.starfarer.api.ui.SectorMapAPI;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;
import src.data.scripts.campaign.GladiatorSociety_TinyFleetFactoryV2;
import src.data.utils.GladiatorSociety_Constants;

public class GladiatorSociety_EventMissionBattle_v2 extends BaseIntelPlugin implements EveryFrameScript, FleetEventListener {

    public static Logger log = Global.getLogger(GladiatorSociety_EventMissionBattle_v2.class);

    private float elapsedDays = 0;
    private boolean ended = false;
    private GladiatorSociety_BountyMission_V2 mission = null;
    private FactionAPI faction;

    private CampaignFleetAPI fleet;

    public float getElapsedDays() {
        return elapsedDays;
    }

    public void setElapsedDays(float elapsedDays) {
        this.elapsedDays = elapsedDays;
    }

    public static PersonBountyEventData getSharedData() {
        return SharedData.getData().getPersonBountyEventData();
    }

    public GladiatorSociety_EventMissionBattle_v2(GladiatorSociety_BountyMission_V2 mission) {
        this.mission = mission;
        faction = Global.getSector().getFaction(GladiatorSociety_Constants.GSFACTION_ID);

        spawnFleet();
        if (isDone()) {
            return;
        }
        SectorAPI sector = Global.getSector();
        FactionAPI player = sector.getFaction(Factions.PLAYER);
        player.setRelationship(GladiatorSociety_Constants.GSFACTION_ID, -1);
        log.info(String.format("Gladiator Battle begin ", mission.getMissionID()));
       // Global.getSector().getIntelManager().queueIntel(this);
       Global.getSector().getIntelManager().addIntel(this, true);

    }

    @Override
    public void reportMadeVisibleToPlayer() {
    }

    @Override
    protected void advanceImpl(float amount) {
        float days = Global.getSector().getClock().convertToDays(amount);
        //days *= 60f;

        elapsedDays += days;

        if ((100 - elapsedDays <= 0) && !isDone()) {
            boolean canEnd = fleet == null || !fleet.isInCurrentLocation();
            if (canEnd) {
                log.info(String.format("Ending Endless Battle"));
                result = new BountyResult(BountyResultType.END_TIME, 0, null);
                sendUpdateIfPlayerHasIntel(result, true);
                cleanUpFleetAndEndIfNecessary();
                return;
            }
        }

        if (fleet == null) {
            return;
        }

        if (fleet.getFlagship() == null || fleet.getFlagship().getCaptain() != mission.getPerson()) {
            result = new BountyResult(BountyResultType.END_OTHER, 0, null);
            boolean current = fleet.isInCurrentLocation();
            sendUpdateIfPlayerHasIntel(result, !current);
            cleanUpFleetAndEndIfNecessary();
        }
    }

    @Override
    public float getTimeRemainingFraction() {
        float f = 1f - elapsedDays / 100;
        return f;
    }

    protected BountyResult result = null;

    @Override
    protected void notifyEnding() {
        super.notifyEnding();
        cleanUpFleetAndEndIfNecessary();
    }

    protected void cleanUpFleetAndEndIfNecessary() {
        if (fleet != null) {
            Misc.makeUnimportant(fleet, "pbe");
            fleet.clearAssignments();
            if (this.mission.getLocation() != null) {
                fleet.getAI().addAssignment(FleetAssignment.GO_TO_LOCATION_AND_DESPAWN, this.mission.getLocation(), 1000000f, null);
            } else {
                fleet.despawn();
            }
            fleet = null; //can't null it because description uses it
        }
        if (!isEnding() && !isEnded()) {
            endAfterDelay();
        }
        mission.getCreator().removeActiveBounty(mission.getMissionID());

    }

    protected boolean willPay() {
        return true;

    }

    protected boolean willRepIncrease() {
        return false;

    }

    @Override
    public void reportBattleOccurred(CampaignFleetAPI fleet, CampaignFleetAPI primaryWinner, BattleAPI battle) {
        if (isDone() || result != null) {
            return;
        }

        if (battle.isInvolved(fleet) && !battle.isPlayerInvolved()) {
            if (fleet.getFlagship() == null || fleet.getFlagship().getCaptain() != mission.getPerson()) {
                fleet.setCommander(fleet.getFaction().createRandomPerson());
                //Global.getSector().reportEventStage(this, "other_end", market.getPrimaryEntity(), messagePriority);
                result = new BountyResult(BountyResultType.END_OTHER, 0, null);
                sendUpdateIfPlayerHasIntel(result, true);
                cleanUpFleetAndEndIfNecessary();
                return;
            }
        }

        CampaignFleetAPI playerFleet = Global.getSector().getPlayerFleet();
        if (!battle.isPlayerInvolved() || !battle.isInvolved(fleet) || battle.onPlayerSide(fleet)) {
            return;
        }

        // didn't destroy the original flagship
        if (fleet.getFlagship() != null && fleet.getFlagship().getCaptain() == mission.getPerson()) {
            return;
        }

        int payment = (int) (mission.getReward() * battle.getPlayerInvolvementFraction());
        if (payment <= 0) {
            result = new BountyResult(BountyResultType.END_OTHER, 0, null);
            sendUpdateIfPlayerHasIntel(result, true);
            cleanUpFleetAndEndIfNecessary();
            return;
        }

        log.info(String.format("Paying salary of %d from Gladiator Society", (int) payment, mission.getMissionID()));
 Global.getSector().getIntelManager().removeIntel(this);
        playerFleet.getCargo().getCredits().add(payment);
        /* ReputationAdjustmentResult rep = Global.getSector().adjustPlayerReputation(
                new RepActionEnvelope(RepActions.PERSON_BOUNTY_REWARD, null, null, null, true, false),
                faction.getId());*/
        result = new BountyResult(BountyResultType.END_PLAYER_BOUNTY, payment, null);
        sendUpdateIfPlayerHasIntel(result, false);

        getSharedData().reportSuccess();
        mission.getCreator().addBountyDone(mission.getMissionID());

        //removeBounty();
        cleanUpFleetAndEndIfNecessary();
    }

    @Override
    public void reportFleetDespawnedToListener(CampaignFleetAPI fleet, FleetDespawnReason reason, Object param) {
        if (isDone() || result != null) {
            return;
        }

        if (this.fleet == fleet) {
            fleet.setCommander(fleet.getFaction().createRandomPerson());
            result = new BountyResult(BountyResultType.END_OTHER, 0, null);
            sendUpdateIfPlayerHasIntel(result, true);
            cleanUpFleetAndEndIfNecessary();
        }
    }

    private void spawnFleet() {

        String fleetName = mission.getPerson().getName().getLast() + "'s" + " Fleet";

        FleetParamsV3 params = new FleetParamsV3(
                null,
                mission.getLocation().getLocationInHyperspace(),
                mission.getBountyFaction(), // quality will always be reduced by non-market-faction penalty, which is what we want 
                null,
                FleetTypes.PERSON_BOUNTY_FLEET,
                mission.getCombatPoints(), // combatPts
                0, // freighterPts 
                0, // tankerPts
                0f, // transportPts
                0f, // linerPts
                0f, // utilityPts
                1f // qualityMod
        );
        params.ignoreMarketFleetSizeMult = true;

        fleet = GladiatorSociety_TinyFleetFactoryV2.createFleet(params, mission);
        if (fleet == null || fleet.isEmpty()) {
            endImmediately();
            return;
        }

        fleet.setCommander(mission.getPerson());
        fleet.getFlagship().setCaptain(mission.getPerson());
        FleetFactoryV3.addCommanderSkills(mission.getPerson(), fleet, null);

        Misc.makeImportant(fleet, GladiatorSociety_Constants.GSFACTION_ID, mission.getDuration());
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);
        fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_NO_MILITARY_RESPONSE, true);

        fleet.setNoFactionInName(true);

        fleet.setFaction(GladiatorSociety_Constants.GSFACTION_ID, true);

        fleet.addEventListener(this);

        LocationAPI location = Global.getSector().getPlayerFleet().getContainingLocation();
        location.addEntity(fleet);
        fleet.setName(fleetName);
        fleet.removeAbility(Abilities.INTERDICTION_PULSE);

        fleet.setLocation(Global.getSector().getPlayerFleet().getLocation().x, Global.getSector().getPlayerFleet().getLocation().y);
        fleet.getAI().addAssignment(FleetAssignment.INTERCEPT, Global.getSector().getPlayerFleet(), 1000000f, null);
        fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_MAKE_AGGRESSIVE, true);
        fleet.getMemoryWithoutUpdate().set("$dialog", mission.getDialog());

    }

    @Override
    public boolean runWhilePaused() {
        return false;
    }

    public static enum BountyResultType {
        END_PLAYER_BOUNTY,
        END_PLAYER_NO_BOUNTY,
        END_PLAYER_NO_REWARD,
        END_OTHER,
        END_TIME,
    }

    public static class BountyResult {

        public BountyResultType type;
        public int payment;
        public ReputationAdjustmentResult rep;

        public BountyResult(BountyResultType type, int payment, ReputationAdjustmentResult rep) {
            this.type = type;
            this.payment = payment;
            this.rep = rep;
        }
    }

    protected void addBulletPoints(TooltipMakerAPI info, IntelInfoPlugin.ListInfoMode mode) {

        Color h = Misc.getHighlightColor();
        Color g = Misc.getGrayColor();
        float pad = 3f;
        float opad = 10f;

        float initPad = pad;
        if (mode == IntelInfoPlugin.ListInfoMode.IN_DESC) {
            initPad = opad;
        }

        Color tc = getBulletColorForMode(mode);

        bullet(info);
        boolean isUpdate = getListInfoParam() != null;

        if (result == null) {
            if (mode == IntelInfoPlugin.ListInfoMode.IN_DESC) {

                info.addPara("%s reward", initPad, tc, h, Misc.getDGSCredits(mission.getReward()));
//				}
                addDays(info, "remaining", 120 - elapsedDays, tc);
            } else {
                if (!isEnding()) {
                    int days = (int) (120 - elapsedDays);
                    String daysStr = "days";
                    if (days <= 1) {
                        days = 1;
                        daysStr = "day";
                    }
                    info.addPara("%s reward, %s " + daysStr + " remaining", 0f, tc,
                            h, Misc.getDGSCredits(mission.getReward()), "" + days);
                }
            }
            unindent(info);
            return;
        }

        switch (result.type) {
            case END_PLAYER_BOUNTY:
                info.addPara("%s received", initPad, tc, h, Misc.getDGSCredits(result.payment));
                break;
            case END_PLAYER_NO_BOUNTY:
                break;
            case END_PLAYER_NO_REWARD:
                break;
            case END_TIME:
                break;
            case END_OTHER:
                break;

        }

        unindent(info);
    }

    @Override
    public void createIntelInfo(TooltipMakerAPI info, IntelInfoPlugin.ListInfoMode mode) {
        Color h = Misc.getHighlightColor();
        Color g = Misc.getGrayColor();
        Color c = getTitleColor(mode);
        float pad = 3f;
        float opad = 10f;

        info.addPara(getName(), c, 0f);

        addBulletPoints(info, mode);

    }

    @Override
    public String getSortString() {
        return "Endless Battle";
    }

    public String getName() {
        String n = mission.getPerson().getName().getFullName();

        if (result != null) {
            switch (result.type) {
                case END_PLAYER_BOUNTY:
                case END_PLAYER_NO_BOUNTY:
                case END_PLAYER_NO_REWARD:
                    return "Battle Completed - " + n;
                case END_OTHER:
                case END_TIME:
                    return "Battle Ended - " + n;
            }
        }

        return "Endless Battle - " + n;
    }

    @Override
    public FactionAPI getFactionForUIColors() {
        return faction;
    }

    @Override
    public String getSmallDescriptionTitle() {
        return getName();
        //return null;
    }

    @Override
    public void createSmallDescription(TooltipMakerAPI info, float width, float height) {
        Color h = Misc.getHighlightColor();
        Color g = Misc.getGrayColor();
        float pad = 3f;
        float opad = 10f;

        //info.addPara(getName(), c, 0f);
        //info.addSectionHeading(getName(), Alignment.MID, 0f);
        info.addImage(mission.getPerson().getPortraitSprite(), width, 128, opad);

        if (result != null) {
            if (result.type == BountyResultType.END_PLAYER_BOUNTY) {
                info.addPara("You have successfully completed this battle.", opad);
            } else if (result.type == BountyResultType.END_PLAYER_NO_BOUNTY) {
                info.addPara("You have successfully completed this battle, but received no "
                        + "credit reward because obvious reason. ", opad);
            } else if (result.type == BountyResultType.END_PLAYER_NO_REWARD) {
                info.addPara("You have successfully completed this bounty, but received no "
                        + "credit reward because obvious reason.", opad);
            } else {
                info.addPara("This battle is no longer on offer. (You are serious?)", opad);
            }
        }

        addBulletPoints(info, IntelInfoPlugin.ListInfoMode.IN_DESC);
        if (result == null) {

            if (mission.getLocation() != null) {
                SectorEntityToken fake = mission.getLocation().getContainingLocation().createToken(0, 0);
                String sheIs = "She is";
                if (mission.getPerson().getGender() == Gender.MALE) {
                    sheIs = "He is";
                }
                info.addPara(sheIs + " where you have requested the battle.", opad);
            }
        }
    }

    @Override
    public String getIcon() {
        return Global.getSettings().getSpriteName("campaignMissions", "gsbounty");
    }

    @Override
    public Set<String> getIntelTags(SectorMapAPI map) {
        Set<String> tags = super.getIntelTags(map);
        tags.add(GladiatorSociety_Constants.ENDLESSBATTLE);
        tags.add(GladiatorSociety_Constants.GSFACTION_ID);
        return tags;
    }

    @Override
    public SectorEntityToken getMapLocation(SectorMapAPI map) {
        Constellation c = mission.getLocation().getConstellation();
        SectorEntityToken entity = null;
        if (c != null && map != null) {
            entity = map.getConstellationLabelEntity(c);
        }
        if (entity == null) {
            entity = mission.getLocation();
        }
        return entity;
    }

}
